// Copyright 2013 Canonical Ltd.  This software is licensed under the
// GNU Lesser General Public License version 3 (see the file COPYING).

package gwacl

import (
    "fmt"
    "math/rand"
)

// pickOne returns a random choice of one of the characters in chars.
func pickOne(chars string) string {
    index := rand.Intn(len(chars))
    return string(chars[index])
}

// makeRandomIdentifier creates an arbitrary identifier of the given length,
// consisting of only ASCII digits and lower-case ASCII letters.
// The identifier will start with the given prefix.  The prefix must be no
// longer than the specified length, or there'll be trouble.
func makeRandomIdentifier(prefix string, length int) string {
    // Only digits and lower-case ASCII letters are accepted.
    const (
        letters = "abcdefghijklmnopqrstuvwxyz"
        digits  = "0123456789"
        chars   = letters + digits
    )

    if len(prefix) > length {
        panic(fmt.Errorf("prefix '%s' is more than the requested %d characters long", prefix, length))
    }

    if len(prefix) == 0 {
        // No prefix.  Still have to start with a letter, so pick one.
        prefix = pickOne(letters)
    }

    id := prefix
    for len(id) < length {
        id += pickOne(chars)
    }
    return id
}

// MakeRandomHostedServiceName generates a pseudo-random name for a hosted
// service, with the given prefix.
//
// The prefix must be as short as possible, be entirely in ASCII, start with
// a lower-case letter, and contain only lower-case letters and digits after
// that.
//
// Be sure your random generator is initialized, or this will always produce
// the same sequence!
func MakeRandomHostedServiceName(prefix string) string {
    // We don't know of any documentation on long a hosted-service name can
    // be, but this is the maximum length that worked in experiments.
    return makeRandomIdentifier(prefix, 63)
}

// MakeRandomHostname generates a pseudo-random hostname for a virtual machine,
// with the given prefix.
//
// The prefix must be as short as possible, be entirely in ASCII, start with
// a lower-case letter, and contain only lower-case letters and digits after
// that.
//
// Be sure your random generator is initialized, or this will always produce
// the same sequence!
func MakeRandomHostname(prefix string) string {
    // Azure documentation says the hostname can be between 1 and 64
    // letters long, but in practice we found it didn't work with anything
    // over 55 letters long.
    return makeRandomIdentifier(prefix, 55)
}
