// Copyright 2013 Canonical Ltd.  This software is licensed under the
// GNU Lesser General Public License version 3 (see the file COPYING).

package gwacl

import (
    . "launchpad.net/gocheck"
    "math/rand"
)

type namesSuite struct{}

var _ = Suite(&namesSuite{})

func (*namesSuite) TestPickOneReturnsOneCharacter(c *C) {
    c.Check(len(pickOne("abcd")), Equals, 1)
}

func (*namesSuite) TestMakeRandomIdentifierObeysLength(c *C) {
    length := 1 + rand.Intn(50)
    c.Check(len(makeRandomIdentifier("x", length)), Equals, length)
}

func (*namesSuite) TestMakeRandomIdentifierRandomizes(c *C) {
    // There is a minute chance that this will start failing just because
    // the randomizer repeats a pattern of results.  If so, seed it.
    c.Check(
        makeRandomIdentifier("x", 100),
        Not(Equals),
        makeRandomIdentifier("x", 100))
}

func (*namesSuite) TestMakeRandomIdentifierPicksDifferentCharacters(c *C) {
    // There is a minute chance that this will start failing just because
    // the randomizer repeats a pattern of results.  If so, seed it.
    chars := make(map[rune]bool)
    for _, chr := range makeRandomIdentifier("", 100) {
        chars[chr] = true
    }
    c.Check(len(chars), Not(Equals), 1)
}

func (*namesSuite) TestMakeRandomIdentifierUsesPrefix(c *C) {
    c.Check(makeRandomIdentifier("prefix", 10), Matches, "prefix.*")
}

func (*namesSuite) TestMakeRandomIdentifierUsesOnlyAcceptedCharacters(c *C) {
    c.Check(makeRandomIdentifier("", 100), Matches, "[0-9a-z]*")
}

func (*namesSuite) TestMakeRandomIdentifierAcceptsEmptyPrefix(c *C) {
    // In particular, the first character must still be a letter.
    c.Check(makeRandomIdentifier("", 1), Matches, "[a-z]")
}
